﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Windows;
using System.Xml.Serialization;

namespace PercentageAggregator.UI.Helper
{
    public class SalesCollection : ObservableCollection<Sale>
    {
        private const string XML_LOCATION = @"/PercentageAggregator.UI;component/Resources/Sales.xml";

        public SalesCollection()
        {
            var sri = Application.GetResourceStream(new Uri(XML_LOCATION, UriKind.RelativeOrAbsolute));
            var serializer = new XmlSerializer(typeof(List<Sale>));

            if (sri != null)
            {
                var sales = serializer.Deserialize(sri.Stream) as List<Sale>;
                if (sales != null)
                    sales.ForEach(Add);
            }
        }
    }

    public class Sale
    {
        public Sale()
        {
            Product = new Product();
            Seller = new Seller();
        }

        public Product Product { get; set; }
        public Seller Seller { get; set; }
        public DateTime Date { get; set; }

        public double Value { get; set; }

        public string Quarter
        {
            get
            {
                return Date.Month < 4
                           ? String.Format(@"Q1 {0}", Date.Year)
                           : (Date.Month < 7
                                  ? String.Format(@"Q2 {0}", Date.Year)
                                  : String.Format(Date.Month < 10
                                                      ? @"Q3 {0}"
                                                      : @"Q4 {0}", Date.Year));
            }
        }

        public string City
        {
            get { return Seller.City; }
            set { Seller.City = value; }
        }

        public int NumberOfUnits { get; set; }

        public double AmountOfSale
        {
            get { return UnitPrice * NumberOfUnits; }
            set { Product.UnitPrice = value / NumberOfUnits; }
        }

        public double UnitPrice
        {
            get { return Product.UnitPrice; }
            set { Product.UnitPrice = value; }
        }
    }

    public class Seller
    {
        public string Name { get; set; }
        public string City { get; set; }
    }

    public class Product
    {
        public string Name { get; set; }
        public double UnitPrice { get; set; }
    }
}