﻿using System.Windows;
using Infragistics.Olap;
using Infragistics.Olap.FlatData;
using PercentageAggregator.UI.Helper;

namespace PercentageAggregator.UI
{
    /// <summary>
    ///     Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();

            var flatDataSource = BuildPivotGridFlatDataSource();

            pivotGrid.DataSource = flatDataSource;
            dataSelector.DataSource = flatDataSource;
        }

        private IOlapViewModel BuildPivotGridFlatDataSource()
        {
            var flatDataSource = new FlatDataSource
                {
                    ItemsSource = new SalesCollection(),
                    Cube = DataSourceBase.GenerateInitialCube(@"Sale"),
                    MeasureListLocation = MeasureListLocation.Columns,
                    DimensionsGenerationMode = DimensionsGenerationMode.Mixed,
                    ParentInFrontForColumns = true
                };

            var cubeMetadata = new CubeMetadata {DisplayName = @"Sample Cube", DataTypeFullName = typeof(Sale).FullName};

            cubeMetadata.DimensionSettings.Add(new DimensionMetadata
                {
                    SourcePropertyName = @"NumberOfUnits",
                    DisplayName = @"Number of Units"
                });

            cubeMetadata.DimensionSettings.Add(new DimensionMetadata
                {
                    SourcePropertyName = @"Product",
                    DisplayName = @"Product"
                });
            cubeMetadata.DimensionSettings.Add(new DimensionMetadata
                {
                    SourcePropertyName = @"City",
                    DisplayName = @"City"
                });
            cubeMetadata.DimensionSettings.Add(new DimensionMetadata
                {
                    SourcePropertyName = @"Seller",
                    DisplayName = @"Seller"
                });
            cubeMetadata.DimensionSettings.Add(new DimensionMetadata
                {
                    SourcePropertyName = @"Date",
                    DisplayName = @"Date"
                });
            cubeMetadata.DimensionSettings.Add(new DimensionMetadata
                {
                    SourcePropertyName = @"Quarter",
                    DisplayName = @"Quarter"
                });
            cubeMetadata.DimensionSettings.Add(new DimensionMetadata
                {
                    SourcePropertyName = @"UnitPrice",
                    DisplayName = @"Unit Price"
                });

            cubeMetadata.DimensionSettings.Add(new DimensionMetadata
                {
                    SourcePropertyName = @"Value",
                    DisplayName = @"Value"
                });

            cubeMetadata.DimensionSettings.Add(new DimensionMetadata
                {
                    SourcePropertyName = @"AmountOfSale",
                    DimensionType = DimensionType.Measure,
                    DisplayName = @"Amount of Sale Sum",
                    DisplayFormat = @"{0:C2}",
                    Aggregator = new DoubleSumAggregator {Identity = @"DoubleSumAmountOfSale"}
                });

            var percentage = new PercentOfTotalAggregator();
            percentage.Parts.Add(new DoubleSumAggregator {Identity = @"DoubleSumAmountOfSale"});

            cubeMetadata.DimensionSettings.Add(new DimensionMetadata
                {
                    DimensionType = DimensionType.Measure,
                    DisplayName = @"Amount Of Sale Percentage",
                    Aggregator = percentage,
                    DisplayFormat = @"{0:f2}%"
                });

            flatDataSource.CubesSettings.Add(cubeMetadata);

            var sellerHierarchy = new HierarchyDescriptor<Sale>(p => p.Seller);
            sellerHierarchy.AddLevel(p =>
                                     @"All Sellers", 
                                     @"All Sellers"
                );
            sellerHierarchy.AddLevel(p =>
                                     p.Seller.Name,
                                     @"Seller Name"
                );
            flatDataSource.AddHierarchyDescriptor(sellerHierarchy);

            var productHierarchy = new HierarchyDescriptor<Sale>(p => p.Product);
            productHierarchy.AddLevel(p =>
                                      @"All Products",
                                      @"All Products"
                );
            productHierarchy.AddLevel(p =>
                                      p.Product.Name,
                                      @"Product Name"
                );
            flatDataSource.AddHierarchyDescriptor(productHierarchy);

            var locationHierarchy = new HierarchyDescriptor<Sale>(p => p.City);
            locationHierarchy.AddLevel(p =>
                                       @"All Locations",
                                       @"All Locations"
                );
            locationHierarchy.AddLevel(p =>
                                       p.City,
                                       @"City"
                );
            flatDataSource.AddHierarchyDescriptor(locationHierarchy);

            return flatDataSource;
        }
    }
}